@@ -35,11 +35,12 @@
 static void iface_modem_init (MMIfaceModem *iface);
 static void iface_modem_3gpp_init (MMIfaceModem3gpp *iface);
 
+static MMIfaceModem *iface_modem_parent;
 static MMIfaceModem3gpp *iface_modem_3gpp_parent;
 
 G_DEFINE_TYPE_EXTENDED (MMBroadbandModemOption, mm_broadband_modem_option, MM_TYPE_BROADBAND_MODEM, 0,
                         G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM, iface_modem_init)
-                        G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM_3GPP, iface_modem_3gpp_init));
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM_3GPP, iface_modem_3gpp_init))
 
 struct _MMBroadbandModemOptionPrivate {
     /* Regex for access-technology related notifications */
@@ -56,11 +57,93 @@ struct _MMBroadbandModemOptionPrivate {
     guint after_power_up_wait_id;
 };
 
+/*****************************************************************************/
+/* Load supported modes (Modem interface) */
+
+static GArray *
+load_supported_modes_finish (MMIfaceModem *self,
+                             GAsyncResult *res,
+                             GError **error)
+{
+    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
+        return NULL;
+
+    return g_array_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
+}
+
+static void
+parent_load_supported_modes_ready (MMIfaceModem *self,
+                                   GAsyncResult *res,
+                                   GSimpleAsyncResult *simple)
+{
+    GError *error = NULL;
+    GArray *all;
+    GArray *combinations;
+    GArray *filtered;
+    MMModemModeCombination mode;
+
+    all = iface_modem_parent->load_supported_modes_finish (self, res, &error);
+    if (!all) {
+        g_simple_async_result_take_error (simple, error);
+        g_simple_async_result_complete (simple);
+        g_object_unref (simple);
+        return;
+    }
+
+    /* Build list of combinations */
+    combinations = g_array_sized_new (FALSE, FALSE, sizeof (MMModemModeCombination), 5);
+
+    /* 2G only */
+    mode.allowed = MM_MODEM_MODE_2G;
+    mode.preferred = MM_MODEM_MODE_NONE;
+    g_array_append_val (combinations, mode);
+    /* 3G only */
+    mode.allowed = MM_MODEM_MODE_3G;
+    mode.preferred = MM_MODEM_MODE_NONE;
+    g_array_append_val (combinations, mode);
+    /* 2G and 3G */
+    mode.allowed = (MM_MODEM_MODE_2G | MM_MODEM_MODE_3G);
+    mode.preferred = MM_MODEM_MODE_NONE;
+    g_array_append_val (combinations, mode);
+    /* 2G and 3G, 2G preferred */
+    mode.allowed = (MM_MODEM_MODE_2G | MM_MODEM_MODE_3G);
+    mode.preferred = MM_MODEM_MODE_2G;
+    g_array_append_val (combinations, mode);
+    /* 2G and 3G, 3G preferred */
+    mode.allowed = (MM_MODEM_MODE_2G | MM_MODEM_MODE_3G);
+    mode.preferred = MM_MODEM_MODE_3G;
+    g_array_append_val (combinations, mode);
+
+    /* Filter out those unsupported modes */
+    filtered = mm_filter_supported_modes (all, combinations);
+    g_array_unref (all);
+    g_array_unref (combinations);
+
+    g_simple_async_result_set_op_res_gpointer (simple, filtered, (GDestroyNotify) g_array_unref);
+    g_simple_async_result_complete (simple);
+    g_object_unref (simple);
+}
+
+static void
+load_supported_modes (MMIfaceModem *self,
+                      GAsyncReadyCallback callback,
+                      gpointer user_data)
+{
+    /* Run parent's loading */
+    iface_modem_parent->load_supported_modes (
+        MM_IFACE_MODEM (self),
+        (GAsyncReadyCallback)parent_load_supported_modes_ready,
+        g_simple_async_result_new (G_OBJECT (self),
+                                   callback,
+                                   user_data,
+                                   load_supported_modes));
+}
+
 /*****************************************************************************/
 /* Load initial allowed/preferred modes (Modem interface) */
 
 static gboolean
-load_allowed_modes_finish (MMIfaceModem *self,
+load_current_modes_finish (MMIfaceModem *self,
                            GAsyncResult *res,
                            MMModemMode *allowed,
                            MMModemMode *preferred,
@@ -119,7 +202,7 @@ load_allowed_modes_finish (MMIfaceModem *self,
 }
 
 static void
-load_allowed_modes (MMIfaceModem *self,
+load_current_modes (MMIfaceModem *self,
                     GAsyncReadyCallback callback,
                     gpointer user_data)
 {
@@ -135,7 +218,7 @@ load_allowed_modes (MMIfaceModem *self,
 /* Set allowed modes (Modem interface) */
 
 static gboolean
-set_allowed_modes_finish (MMIfaceModem *self,
+set_current_modes_finish (MMIfaceModem *self,
                           GAsyncResult *res,
                           GError **error)
 {
@@ -160,7 +243,7 @@ allowed_mode_update_ready (MMBroadbandModemOption *self,
 }
 
 static void
-set_allowed_modes (MMIfaceModem *self,
+set_current_modes (MMIfaceModem *self,
                    MMModemMode allowed,
                    MMModemMode preferred,
                    GAsyncReadyCallback callback,
@@ -173,7 +256,7 @@ set_allowed_modes (MMIfaceModem *self,
     result = g_simple_async_result_new (G_OBJECT (self),
                                         callback,
                                         user_data,
-                                        set_allowed_modes);
+                                        set_current_modes);
 
     if (allowed == MM_MODEM_MODE_2G)
         option_mode = 0;
@@ -186,7 +269,8 @@ set_allowed_modes (MMIfaceModem *self,
             option_mode = 3;
         else /* none preferred, so AUTO */
             option_mode = 5;
-    }
+    } else if (allowed == MM_MODEM_MODE_ANY && preferred == MM_MODEM_MODE_NONE)
+        option_mode = 5;
 
     if (option_mode < 0) {
         gchar *allowed_str;
@@ -591,7 +675,7 @@ after_power_up_wait_cb (GSimpleAsyncResult *result)
     option->priv->after_power_up_wait_id = 0;
     g_object_unref (option);
 
-    return FALSE;
+    return G_SOURCE_REMOVE;
 }
 
 static void
@@ -662,7 +746,7 @@ modem_3gpp_load_imei (MMIfaceModem3gpp *self,
 /* Setup/Cleanup unsolicited events (3GPP interface) */
 
 static void
-option_ossys_tech_changed (MMAtSerialPort *port,
+option_ossys_tech_changed (MMPortSerialAt *port,
                            GMatchInfo *info,
                            MMBroadbandModemOption *self)
 {
@@ -699,7 +783,7 @@ option_ossys_tech_changed (MMAtSerialPort *port,
 }
 
 static void
-option_2g_tech_changed (MMAtSerialPort *port,
+option_2g_tech_changed (MMPortSerialAt *port,
                         GMatchInfo *match_info,
                         MMBroadbandModemOption *self)
 {
@@ -715,7 +799,7 @@ option_2g_tech_changed (MMAtSerialPort *port,
 }
 
 static void
-option_3g_tech_changed (MMAtSerialPort *port,
+option_3g_tech_changed (MMPortSerialAt *port,
                         GMatchInfo *match_info,
                         MMBroadbandModemOption *self)
 {
@@ -731,7 +815,7 @@ option_3g_tech_changed (MMAtSerialPort *port,
 }
 
 static void
-option_signal_changed (MMAtSerialPort *port,
+option_signal_changed (MMPortSerialAt *port,
                        GMatchInfo *match_info,
                        MMBroadbandModemOption *self)
 {
@@ -759,7 +843,7 @@ static void
 set_unsolicited_events_handlers (MMBroadbandModemOption *self,
                                  gboolean enable)
 {
-    MMAtSerialPort *ports[2];
+    MMPortSerialAt *ports[2];
     guint i;
 
     ports[0] = mm_base_modem_peek_port_primary (MM_BASE_MODEM (self));
@@ -771,36 +855,36 @@ set_unsolicited_events_handlers (MMBroadbandModemOption *self,
             continue;
 
         /* Access technology related */
-        mm_at_serial_port_add_unsolicited_msg_handler (
+        mm_port_serial_at_add_unsolicited_msg_handler (
             ports[i],
             self->priv->_ossysi_regex,
-            enable ? (MMAtSerialUnsolicitedMsgFn)option_ossys_tech_changed : NULL,
+            enable ? (MMPortSerialAtUnsolicitedMsgFn)option_ossys_tech_changed : NULL,
             enable ? self : NULL,
             NULL);
-        mm_at_serial_port_add_unsolicited_msg_handler (
+        mm_port_serial_at_add_unsolicited_msg_handler (
             ports[i],
             self->priv->_octi_regex,
-            enable ? (MMAtSerialUnsolicitedMsgFn)option_2g_tech_changed : NULL,
+            enable ? (MMPortSerialAtUnsolicitedMsgFn)option_2g_tech_changed : NULL,
             enable ? self : NULL,
             NULL);
-        mm_at_serial_port_add_unsolicited_msg_handler (
+        mm_port_serial_at_add_unsolicited_msg_handler (
             ports[i],
             self->priv->_ouwcti_regex,
-            enable ? (MMAtSerialUnsolicitedMsgFn)option_3g_tech_changed : NULL,
+            enable ? (MMPortSerialAtUnsolicitedMsgFn)option_3g_tech_changed : NULL,
             enable ? self : NULL,
             NULL);
 
         /* Signal quality related */
-        mm_at_serial_port_add_unsolicited_msg_handler (
+        mm_port_serial_at_add_unsolicited_msg_handler (
             ports[i],
             self->priv->_osigq_regex,
-            enable ? (MMAtSerialUnsolicitedMsgFn)option_signal_changed : NULL,
+            enable ? (MMPortSerialAtUnsolicitedMsgFn)option_signal_changed : NULL,
             enable ? self : NULL,
             NULL);
 
         /* Other unsolicited events to always ignore */
         if (!enable)
-            mm_at_serial_port_add_unsolicited_msg_handler (
+            mm_port_serial_at_add_unsolicited_msg_handler (
                 ports[i],
                 self->priv->ignore_regex,
                 NULL, NULL, NULL);
@@ -827,7 +911,7 @@ parent_setup_unsolicited_events_ready (MMIfaceModem3gpp *self,
     else {
         /* Our own setup now */
         set_unsolicited_events_handlers (MM_BROADBAND_MODEM_OPTION (self), TRUE);
-        g_simple_async_result_set_op_res_gboolean (G_SIMPLE_ASYNC_RESULT (res), TRUE);
+        g_simple_async_result_set_op_res_gboolean (simple, TRUE);
     }
 
     g_simple_async_result_complete (simple);
@@ -863,7 +947,7 @@ parent_cleanup_unsolicited_events_ready (MMIfaceModem3gpp *self,
     if (!iface_modem_3gpp_parent->cleanup_unsolicited_events_finish (self, res, &error))
         g_simple_async_result_take_error (simple, error);
     else
-        g_simple_async_result_set_op_res_gboolean (G_SIMPLE_ASYNC_RESULT (res), TRUE);
+        g_simple_async_result_set_op_res_gboolean (simple, TRUE);
     g_simple_async_result_complete (simple);
     g_object_unref (simple);
 }
@@ -1064,6 +1148,21 @@ setup_ports (MMBroadbandModem *self)
 
 /*****************************************************************************/
 
+static gboolean
+is_nozomi (const gchar **drivers)
+{
+    if (drivers) {
+        guint i;
+
+        for (i = 0; drivers[i]; i++) {
+            if (g_str_equal (drivers[i], "nozomi"))
+                return TRUE;
+        }
+    }
+
+    return FALSE;
+}
+
 MMBroadbandModemOption *
 mm_broadband_modem_option_new (const gchar *device,
                                const gchar **drivers,
@@ -1071,12 +1170,18 @@ mm_broadband_modem_option_new (const gchar *device,
                                guint16 vendor_id,
                                guint16 product_id)
 {
+    MMModem3gppFacility ignored;
+
+    /* Ignore PH-SIM facility in 'nozomi' managed modems */
+    ignored = is_nozomi (drivers) ? MM_MODEM_3GPP_FACILITY_PH_SIM : MM_MODEM_3GPP_FACILITY_NONE;
+
     return g_object_new (MM_TYPE_BROADBAND_MODEM_OPTION,
                          MM_BASE_MODEM_DEVICE, device,
                          MM_BASE_MODEM_DRIVERS, drivers,
                          MM_BASE_MODEM_PLUGIN, plugin,
                          MM_BASE_MODEM_VENDOR_ID, vendor_id,
                          MM_BASE_MODEM_PRODUCT_ID, product_id,
+                         MM_IFACE_MODEM_3GPP_IGNORED_FACILITY_LOCKS, ignored,
                          NULL);
 }
 
@@ -1098,7 +1203,7 @@ static void
 mm_broadband_modem_option_init (MMBroadbandModemOption *self)
 {
     /* Initialize private data */
-    self->priv = G_TYPE_INSTANCE_GET_PRIVATE ((self),
+    self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
                                               MM_TYPE_BROADBAND_MODEM_OPTION,
                                               MMBroadbandModemOptionPrivate);
     self->priv->after_power_up_wait_id = 0;
@@ -1119,14 +1224,18 @@ mm_broadband_modem_option_init (MMBroadbandModemOption *self)
 static void
 iface_modem_init (MMIfaceModem *iface)
 {
+    iface_modem_parent = g_type_interface_peek_parent (iface);
+
     iface->modem_after_power_up = modem_after_power_up;
     iface->modem_after_power_up_finish = modem_after_power_up_finish;
     iface->load_access_technologies = load_access_technologies;
     iface->load_access_technologies_finish = load_access_technologies_finish;
-    iface->load_allowed_modes = load_allowed_modes;
-    iface->load_allowed_modes_finish = load_allowed_modes_finish;
-    iface->set_allowed_modes = set_allowed_modes;
-    iface->set_allowed_modes_finish = set_allowed_modes_finish;
+    iface->load_supported_modes = load_supported_modes;
+    iface->load_supported_modes_finish = load_supported_modes_finish;
+    iface->load_current_modes = load_current_modes;
+    iface->load_current_modes_finish = load_current_modes_finish;
+    iface->set_current_modes = set_current_modes;
+    iface->set_current_modes_finish = set_current_modes_finish;
 }
 
 static void
