@@ -150,7 +150,7 @@ cpms_timeout_cb (ModemAfterSimUnlockContext *ctx)
 {
     ctx->retries--;
     modem_after_sim_unlock_context_step (ctx);
-    return FALSE;
+    return G_SOURCE_REMOVE;
 }
 
 static void
@@ -633,7 +633,7 @@ parent_setup_unsolicited_events_ready (MMIfaceModem3gpp *self,
         mm_common_zte_set_unsolicited_events_handlers (MM_BROADBAND_MODEM (self),
                                                        MM_BROADBAND_MODEM_ZTE (self)->priv->unsolicited_setup,
                                                        TRUE);
-        g_simple_async_result_set_op_res_gboolean (G_SIMPLE_ASYNC_RESULT (res), TRUE);
+        g_simple_async_result_set_op_res_gboolean (simple, TRUE);
     }
 
     g_simple_async_result_complete (simple);
@@ -669,7 +669,7 @@ parent_cleanup_unsolicited_events_ready (MMIfaceModem3gpp *self,
     if (!iface_modem_3gpp_parent->cleanup_unsolicited_events_finish (self, res, &error))
         g_simple_async_result_take_error (simple, error);
     else
-        g_simple_async_result_set_op_res_gboolean (G_SIMPLE_ASYNC_RESULT (res), TRUE);
+        g_simple_async_result_set_op_res_gboolean (simple, TRUE);
     g_simple_async_result_complete (simple);
     g_object_unref (simple);
 }
